@@ -33,6 +33,7 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "runMode", &runMode));
     parArray.push_back(new ParameterInfoScalar <int> (-1, -1, "runThreadN", &runThreadN));        
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "runDirPerm", &runDirPermIn));
+    parArray.push_back(new ParameterInfoScalar <int> (-1, -1, "runRNGseed", &runRNGseed));        
     
     //genome
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "genomeDir", &genomeDir));
@@ -46,7 +47,9 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "readFilesIn", &readFilesIn));
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "readFilesCommand", &readFilesCommand));
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "readMatesLengthsIn", &readMatesLengthsIn));
-    parArray.push_back(new ParameterInfoScalar <uint> (-1, -1, "readMapNumber", &readMapNumber));        
+    parArray.push_back(new ParameterInfoScalar <uint> (-1, -1, "readMapNumber", &readMapNumber));     
+    parArray.push_back(new ParameterInfoVector <string> (-1, -1, "readNameSeparator", &readNameSeparator));
+
     
     //input from BAM
     parArray.push_back(new ParameterInfoScalar <string> (-1, -1, "inputBAMfile", &inputBAMfile));
@@ -71,6 +74,7 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <string>     (-1, 2, "outStd", &outStd));        
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "outReadsUnmapped", &outReadsUnmapped));
     parArray.push_back(new ParameterInfoScalar <int>        (-1, -1, "outQSconversionAdd", &outQSconversionAdd));
+    parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "outMultimapperOrder", &outMultimapperOrder.mode));
     
     //outSAM
     parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "outSAMtype", &outSAMtype));    
@@ -90,6 +94,9 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "outSAMheaderCommentFile", &outSAMheaderCommentFile));
     parArray.push_back(new ParameterInfoScalar <int>        (-1, -1, "outBAMcompression", &outBAMcompression));
     parArray.push_back(new ParameterInfoScalar <int>        (-1, -1, "outBAMsortingThreadN", &outBAMsortingThreadN));
+    parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "outSAMfilter", &outSAMfilter.mode));
+    parArray.push_back(new ParameterInfoScalar <uint>     (-1, -1, "outSAMmultNmax", &outSAMmultNmax));
+    parArray.push_back(new ParameterInfoScalar <uint>     (-1, -1, "outSAMattrIHstart", &outSAMattrIHstart));
 
 
    //output SJ filtering
@@ -166,12 +173,13 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignTranscriptsPerReadNmax", &alignTranscriptsPerReadNmax));    
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignSJoverhangMin", &alignSJoverhangMin));    
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignSJDBoverhangMin", &alignSJDBoverhangMin));    
-
+    parArray.push_back(new ParameterInfoVector <int32>      (-1, -1, "alignSJstitchMismatchNmax", &alignSJstitchMismatchNmax));
+    
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignSplicedMateMapLmin", &alignSplicedMateMapLmin));       
     parArray.push_back(new ParameterInfoScalar <double>     (-1, -1, "alignSplicedMateMapLminOverLmate", &alignSplicedMateMapLminOverLmate));       
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignWindowsPerReadNmax", &alignWindowsPerReadNmax));  
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "alignTranscriptsPerWindowNmax", &alignTranscriptsPerWindowNmax));  
-    parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "alignEndsType", &alignEndsType));  
+    parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "alignEndsType", &alignEndsType.in));  
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "alignSoftClipAtReferenceEnds", &alignSoftClipAtReferenceEnds));  
 
 
@@ -183,6 +191,8 @@ Parameters::Parameters() {//initalize parameters info
     parArray.push_back(new ParameterInfoScalar <int>        (-1, -1, "chimScoreJunctionNonGTAG", &chimScoreJunctionNonGTAG));    
     parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "chimJunctionOverhangMin", &chimJunctionOverhangMin));    
     parArray.push_back(new ParameterInfoScalar <string>     (-1, -1, "chimOutType", &chimOutType));
+    parArray.push_back(new ParameterInfoVector <string>     (-1, -1, "chimFilter", &chimFilter));
+    parArray.push_back(new ParameterInfoScalar <uint>       (-1, -1, "chimSegmentReadGapMax", &chimSegmentReadGapMax));
     
     //sjdb
     parArray.push_back(new ParameterInfoVector <string> (-1, -1, "sjdbFileChrStartEnd", &sjdbFileChrStartEnd));
@@ -251,7 +261,16 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
                 exit(0);
             };
             
-            if (oneArg.substr(0,2)=="--") {//parameter name, cut -- 
+            size_t found = oneArg.find("=");
+            if (found!=string::npos && oneArg.substr(0,2)=="--") {// --parameter=value
+                string key = oneArg.substr(2, found - 2);
+                string val = oneArg.substr(found + 1);
+                if (val.find_first_of(" \t")!=std::string::npos) {//there is white space in the argument, put "" around
+                    val ='\"' + val + '\"';
+                };                
+                commandLineFile += '\n' + key + ' ' + val;
+            }
+            else if (oneArg.substr(0,2)=="--") {//parameter name, cut -- 
                 commandLineFile +='\n' + oneArg.substr(2);
             } else {//parameter value
                 if (oneArg.find_first_of(" \t")!=std::string::npos) {//there is white space in the argument, put "" around
@@ -368,8 +387,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
     
     ///////////////////////////////////////// Old variables
     outputBED[0]=0; outputBED[1]=0; //controls the format of starBED output. Will be replaced with HDF5 output
-    //chimeric
-    maxChimReadGap=0;
+
     //annot scoring
     annotScoreScale=0;
     annotSignalFile="-";
@@ -600,6 +618,10 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
     for (int ii=0;ii<4;ii++) {
         if (outSJfilterOverhangMin.at(ii)<0) outSJfilterOverhangMin.at(ii)=numeric_limits<int32>::max();
         if (outSJfilterCountUniqueMin.at(ii)<0) outSJfilterCountUniqueMin.at(ii)=numeric_limits<int32>::max();
+        if (outSJfilterCountTotalMin.at(ii)<0) outSJfilterCountTotalMin.at(ii)=numeric_limits<int32>::max();
+        if (outSJfilterDistToOtherSJmin.at(ii)<0) outSJfilterDistToOtherSJmin.at(ii)=numeric_limits<int32>::max();
+        
+        if (alignSJstitchMismatchNmax.at(ii)<0) alignSJstitchMismatchNmax.at(ii)=numeric_limits<int32>::max();
     };
     
     if (limitGenomeGenerateRAM==0) {//must be >0
@@ -632,6 +654,35 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
         };    
     };
     
+    outSAMfilter.yes=false;    
+    outSAMfilter.KeepOnlyAddedReferences=false;
+    if (outSAMfilter.mode.at(0)=="KeepOnlyAddedReferences")
+    {
+        outSAMfilter.yes=true;
+        outSAMfilter.KeepOnlyAddedReferences=true;
+    } else if (outSAMfilter.mode.at(0)!="None")
+    {
+        ostringstream errOut;
+        errOut <<"EXITING because of FATAL INPUT ERROR: unknown/unimplemented value for --outSAMfilter: "<<outSAMfilter.mode.at(0) <<"\n";
+        errOut <<"SOLUTION: specify one of the allowed values: KeepOnlyAddedReferences or None\n";
+        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);                
+    };
+        
+    if (outMultimapperOrder.mode=="Old_2.4")
+    {
+        outMultimapperOrder.random=false;      
+    } else if (outMultimapperOrder.mode=="Random")
+    {
+        outMultimapperOrder.random=true;
+    } else
+    {
+        ostringstream errOut;
+        errOut <<"EXITING because of FATAL INPUT ERROR: unknown/unimplemented value for --outMultimapperOrder: "<<outMultimapperOrder.mode <<"\n";
+        errOut <<"SOLUTION: specify one of the allowed values: Old_2.4 or SortedByCoordinate or Random\n";
+        exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);                
+    };
+        
+    
     readNmates=readFilesIn.size(); //for now the number of mates is defined by the number of input files
     
     if (runMode=="alignReads" && genomeLoad!="Remove" && genomeLoad!="LoadAndExit") {//open reads files to check if they are present
@@ -812,16 +863,31 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
        inOut->logMain << "WARNING --chimOutType=WithinBAM, therefore STAR will output NM attribute" <<endl;
     };
 
-    
-    outFilterMismatchNoverLmax1=outFilterMismatchNoverLmax;
-    if (alignEndsType=="EndToEnd") {
-        outFilterMismatchNoverLmax1=-1;
-    } else if (alignEndsType=="Local" || alignEndsType=="Extend5pOfRead1" ) {
+    alignEndsType.ext[0][0]=false;
+    alignEndsType.ext[0][1]=false;
+    alignEndsType.ext[1][0]=false;
+    alignEndsType.ext[1][1]=false;
+
+    if (alignEndsType.in=="EndToEnd") 
+    {
+        alignEndsType.ext[0][0]=true;
+        alignEndsType.ext[0][1]=true;
+        alignEndsType.ext[1][0]=true;
+        alignEndsType.ext[1][1]=true;
+    } else if (alignEndsType.in=="Extend5pOfRead1" ) 
+    {
+        alignEndsType.ext[0][0]=true;
+    } else if (alignEndsType.in=="Extend3pOfRead1" )
+    {
+        alignEndsType.ext[0][1]=true;
+    } else if (alignEndsType.in=="Local")
+    {
         //nothing to do for now
-    } else {
+    } else 
+    {
         ostringstream errOut;
-        errOut <<"EXITING because of FATAL INPUT ERROR: unknown/unimplemented value for --alignEndsType: "<<alignEndsType <<"\n";
-        errOut <<"SOLUTION: re-run STAR with --alignEndsType Local or EndToEnd\n";
+        errOut <<"EXITING because of FATAL INPUT ERROR: unknown/unimplemented value for --alignEndsType: "<<alignEndsType.in <<"\n";
+        errOut <<"SOLUTION: re-run STAR with --alignEndsType Local OR EndToEnd OR Extend5pOfRead1 OR Extend3pOfRead1\n";
         exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
         
@@ -893,17 +959,27 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
     {
         ostringstream errOut;
         errOut << "EXITING because of fatal PARAMETERS error: --twopass1readsN is defined, but --twoPassMode is not defined\n";
-        errOut << "SOLUTION: to activate the 2-pass mode, use --twoPassMode Basic";
+        errOut << "SOLUTION: to activate the 2-pass mode, use --twopassMode Basic";
         exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
     
+    
+    
     twoPass.yes=false;
     if (twoPass.mode!="None") {//2-pass parameters
+        if (runMode!="alignReads")
+        {
+            ostringstream errOut;
+            errOut << "EXITING because of fatal PARAMETERS error: 2-pass mapping option  can only be used with --runMode alignReads\n";
+            errOut << "SOLUTION: remove --twopassMode option";
+            exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+        };
+        
         if (twoPass.mode!="Basic")
         {
             ostringstream errOut;
             errOut << "EXITING because of fatal PARAMETERS error: unrecognized value of --twopassMode="<<twoPass.mode<<"\n";
-            errOut << "SOLUTION: for the 2-pass mode, use allowes values --twopassMode: Basic";
+            errOut << "SOLUTION: for the 2-pass mode, use allowed values --twopassMode: Basic";
             exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
         };
         
@@ -936,6 +1012,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
 
     sjdbInsert.pass1=false;
     sjdbInsert.pass2=false;
+    sjdbInsert.yes=false;
     if (sjdbFileChrStartEnd.at(0)!="-" || sjdbGTFfile!="-")
     {//will insert annotated sjdb on the fly
        sjdbInsert.pass1=true;
@@ -983,7 +1060,7 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             errOut <<"SOLUTION: please use default --genomeLoad NoSharedMemory, \n        OR specify --limitBAMsortRAM the amount of RAM (bytes) that can be allocated for BAM sorting in addition to shared memory allocated for the genome.\n        --limitBAMsortRAM typically has to be > 10000000000 (i.e 10GB).\n";
             exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
         };
-        inOut->logMain<<"WARNING: --limitBAMsortRAM=0, will use genome sizeas RAM linit foro BAM sorting\n";
+        inOut->logMain<<"WARNING: --limitBAMsortRAM=0, will use genome size as RAM limit for BAM sorting\n";
     };
 
     if (chimOutType=="WithinBAM" && !outBAMunsorted && !outBAMcoord) {
@@ -993,6 +1070,53 @@ void Parameters::inputParameters (int argInN, char* argIn[]) {//input parameters
             exitWithError(errOut.str(), std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
     };
     
+    //chimeric
+    chimPar.filter.genomicN=false;
+    for (int ii=0; ii<chimFilter.size(); ii++)
+    {
+        if (chimFilter.at(ii)=="banGenomicN")
+        {
+            chimPar.filter.genomicN=true;
+        } 
+        else if (chimFilter.at(ii)=="None")
+        {//nothing to do
+        }
+        else 
+        {
+            ostringstream errOut;
+            errOut << "EXITING because of fatal PARAMETERS error: unrecognized value of --chimFilter="<<chimFilter.at(ii)<<"\n";
+            errOut << "SOLUTION: use allowed values: banGenomicN || None";
+            exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+        };
+    };
+    
+    for (int ii=0; ii<readNameSeparator.size(); ii++)
+    {
+        if (readNameSeparator.at(ii)=="space")
+        {
+            readNameSeparatorChar.push_back(' ');
+        } 
+        else if (readNameSeparator.at(ii)=="none")
+        {
+            //nothing to do
+        }
+        else if (readNameSeparator.at(ii).size()==1)
+        {
+            readNameSeparatorChar.push_back(readNameSeparator.at(ii).at(0));
+        }
+        else 
+        {
+            ostringstream errOut;
+            errOut << "EXITING because of fatal PARAMETERS error: unrecognized value of --readNameSeparator="<<readNameSeparator.at(ii)<<"\n";
+            errOut << "SOLUTION: use allowed values: space OR single characters";
+            exitWithError(errOut.str(),std::cerr, inOut->logMain, EXIT_CODE_PARAMETER, *this);
+        };    
+    };
+
+
+    //genome parameters
+    genomeChrBinNbases=1LLU<<genomeChrBinNbits;
+    
     inOut->logMain << "Finished loading and checking parameters\n" <<flush;
 };
 
@@ -1136,7 +1260,6 @@ void Parameters::chrInfoLoad() {//find chrStart,Length,nChr from Genome G
 
 //////////////////////////////////////////////////////////
 void Parameters::chrBinFill() {
-    genomeChrBinNbases=1LLU<<genomeChrBinNbits;
     chrBinN = chrStart[nChrReal]/genomeChrBinNbases+1;    
     chrBin = new uint [chrBinN];
     for (uint ii=0, ichr=1; ii<chrBinN; ++ii) {
