@@ -11,6 +11,8 @@
 #include "sjdbPrepare.h"
 #include "genomeParametersWrite.h"
 #include "sjdbInsertJunctions.h"
+#include "genomeScanFastaFiles.h"
+#include "genomeSAindex.h"
 
 #include "serviceFuns.cpp"
 #include "streamFuns.h"
@@ -20,47 +22,70 @@ uint globalL;
 
 
 inline int funCompareSuffixes ( const void *a, const void *b){
-	uint jj=0LLU;
         
-	uint *ga=(uint*)((globalG-7LLU)+(*((uint*)a)));
-	uint *gb=(uint*)((globalG-7LLU)+(*((uint*)b)));
+    uint *ga=(uint*)((globalG-7LLU)+(*((uint*)a)));
+    uint *gb=(uint*)((globalG-7LLU)+(*((uint*)b)));
+
+    uint jj=0;
+    int  ii=0;
     uint va=0,vb=0;
+    uint8 *va1, *vb1;
 
-	while (va==vb && jj<globalL) {
-		va=*(ga-jj);
-		vb=*(gb-jj);
-		jj++;
-	};
+    bool aeqb=true;
+    while (aeqb) {
+        va=*(ga-jj);
+        vb=*(gb-jj);
+        va1=(uint8*) &va;
+        vb1=(uint8*) &vb;
+        for (ii=7;ii>=0;ii--)
+        {
+            if (va1[ii]!=vb1[ii] || va1[ii]==5)
+            {
+                aeqb=false;
+                break;
+            };
+        };
+        jj++;
+    };
 
-	if (va>vb) {
-		return 1;
-	} else if (va==vb) {
-		return 0;
-	} else {
-		return -1;
-	};
+    if (va1[ii]>vb1[ii]) 
+    {
+        return 1;
+    } else if (va1[ii]<vb1[ii]) 
+    {
+        return -1;
+    } else 
+    {//va=vb at the end of chr
+        if ( *((uint*)a) > *((uint*)b) )
+        {//anti-stable order,since indexes are sorted in the reverse order 
+            return  -1;
+        } else
+        {//a cannot be equal to b
+            return 1;
+        };
+    };
 };
 
-inline bool funCompareSuffixesBool ( const void *a, const void *b) 
-{
-	uint jj=0LLU;
-        
-	uint *ga=(uint*)((globalG-7LLU)+(*((uint*)a)));
-	uint *gb=(uint*)((globalG-7LLU)+(*((uint*)b)));
-    uint va=0,vb=0;
-
-	while (va==vb && jj<globalL) {
-		va=*(ga-jj);
-		vb=*(gb-jj);
-		jj++;
-	};
-
-	if (va<vb) {
-        return true;
-	} else {
-		return false;
-	};
-};
+// inline bool funCompareSuffixesBool ( const void *a, const void *b) 
+// {
+// 	uint jj=0LLU;
+//         
+// 	uint *ga=(uint*)((globalG-7LLU)+(*((uint*)a)));
+// 	uint *gb=(uint*)((globalG-7LLU)+(*((uint*)b)));
+//     uint va=0,vb=0;
+// 
+// 	while (va==vb && jj<globalL) {
+// 		va=*(ga-jj);
+// 		vb=*(gb-jj);
+// 		jj++;
+// 	};
+// 
+// 	if (va<vb) {
+//         return true;
+// 	} else {
+// 		return false;
+// 	};
+// };
 
 
 inline uint funG2strLocus (uint SAstr, uint const N, char const GstrandBit, uint const GstrandMask) {
@@ -70,62 +95,6 @@ inline uint funG2strLocus (uint SAstr, uint const N, char const GstrandBit, uint
     return SAstr;
 };
 
-uint genomeScanFastaFiles (Parameters *P, char* G, bool flagRun) {//scans fasta files. flagRun=false: check and find full size, flaRun=true: collect all the data
-    uint N=0; //total number of bases in the genome, including chr "spacers"
-    ifstream fileIn;
-    for (uint ii=0;ii<P->genomeFastaFiles.size();ii++) {//all the input files
-        fileIn.open(P->genomeFastaFiles.at(ii).c_str());
-        if ( !fileIn.good() ) {//
-            ostringstream errOut;
-            errOut << "EXITING because of INPUT ERROR: could not open genomeFastaFile: " <<P->genomeFastaFiles.at(ii) <<"\n";
-            exitWithError(errOut.str(),std::cerr, P->inOut->logMain, EXIT_CODE_INPUT_FILES, *P);            
-        };
-        while(!fileIn.eof()) {//read each file until eof
-            string lineIn (4096,'.');
-            getline(fileIn,lineIn);
-            if (lineIn[0]=='>') {//new chromosome
-                if (!flagRun) {
-                    istringstream lineInStream (lineIn);
-                    lineInStream.ignore(1,' ');
-                    string chrName1;
-                    lineInStream >> chrName1;
-                    P->chrName.push_back(chrName1);
-                };
-                
-                if (!flagRun && P->chrStart.size()>0) P->chrLength.push_back(N-P->chrStart.at(P->chrStart.size()-1)); //true length of the chr  
-                
-                if (N>0) {//pad the chromosomes to bins boudnaries
-                    N = ( (N+1)/P->genomeChrBinNbases+1 )*P->genomeChrBinNbases;
-                };
-
-                if (!flagRun) {
-                    P->chrStart.push_back(N);    
-                    P->inOut->logMain << P->genomeFastaFiles.at(ii)<<" : chr # " << P->chrStart.size()-1 << "  \""<<P->chrName.at(P->chrStart.size()-1)<<"\" chrStart: "<<N<<"\n"<<flush;
-                };
-            } else {//char lines
-                if (flagRun) lineIn.copy(G+N,lineIn.size(),0);
-                N += lineIn.size();
-            };
-        };
-        fileIn.close();        
-    };
-    
-   
-    if (!flagRun) P->chrLength.push_back(N-P->chrStart.at(P->chrStart.size()-1)); //true length of the chr  
-
-    N = ( (N+1)/P->genomeChrBinNbases+1)*P->genomeChrBinNbases;
-        
-    if (!flagRun) { 
-        P->nChrReal=P->chrStart.size();
-        P->chrStart.push_back(N); //last chromosome end
-        for (uint ii=0;ii<P->nChrReal;ii++) {
-            P->chrNameIndex[P->chrName[ii]]=ii;
-        };
-    };
-    
-    return N;
-};        
-
 void genomeGenerate(Parameters *P) {
     
     //check parameters
@@ -161,7 +130,7 @@ void genomeGenerate(Parameters *P) {
     //define some parameters from input parameters
     P->genomeChrBinNbases=1LLU << P->genomeChrBinNbits;
     //write genome parameters file
-    genomeParametersWrite(P->genomeDir+("/genomeParameters.txt"), P, "ERROR_00102");
+    genomeParametersWrite(P->genomeDir+("/genomeParameters.txt"), P, ERROR_OUT);
     
     char *G=NULL, *G1=NULL;        
     uint nGenomeReal=genomeScanFastaFiles(P,G,false);//first scan the fasta file to find all the sizes  
@@ -174,37 +143,18 @@ void genomeGenerate(Parameters *P) {
     
     memset(G1,GENOME_spacingChar,nG1alloc);//initialize to K-1 all bytes
  
-    genomeScanFastaFiles(P,G,true);    //load the genome sequence
-     
-    //convert the genome to 0,1,2,3,4
-    for (uint jj=0;jj<nGenomeReal;jj++) {
-        switch (int(G[jj])){
-            case(65): case(97):  G[jj]=char(0);break;//A
-            case(67): case(99):  G[jj]=char(1);break;//C           
-            case(71): case(103): G[jj]=char(2);break;//G                       
-            case(84): case(116): G[jj]=char(3);break;//T                                
-            case(78): case(110): G[jj]=char(4);break;//N
-            case(48):            G[jj]=GENOME_spacingChar;break;//chromosomal breaks within the sequences
-            default:              //anything else
-                if (G[jj]!=GENOME_spacingChar) {
-//                     P->inOut->logMain << "Unexpected character: char="<< G[jj] << "   int="<<int(G[jj])<<"   at " << jj << " , replacing with N\n";
-                     G[jj]=char(4);                                 
-                };
-        };
-    };    
+    genomeScanFastaFiles(P,G,true);    //load the genome sequence   
 
     uint N = nGenomeReal;
     P->nGenome=N;
     uint N2 = N*2;     
 
-    ofstream chrN,chrS,chrL,chrNL;
+    ofstream & chrN = ofstrOpen(P->genomeDir+"/chrName.txt",ERROR_OUT, P);
+    ofstream & chrS = ofstrOpen(P->genomeDir+"/chrStart.txt",ERROR_OUT, P);
+    ofstream & chrL = ofstrOpen(P->genomeDir+"/chrLength.txt",ERROR_OUT, P);
+    ofstream & chrNL = ofstrOpen(P->genomeDir+"/chrNameLength.txt",ERROR_OUT, P);
     
-    ofstrOpen(P->genomeDir+"/chrName.txt","ERROR_00103", P, chrN);   
-    ofstrOpen(P->genomeDir+"/chrStart.txt","ERROR_00103", P, chrS);   
-    ofstrOpen(P->genomeDir+"/chrLength.txt","ERROR_00103", P, chrL);   
-    ofstrOpen(P->genomeDir+"/chrNameLength.txt","ERROR_00103", P, chrNL);   
-    
-    for (uint ii=0;ii<P->nChrReal;ii++) {//output names, starts, lengths               
+    for (uint ii=0;ii<P->nChrReal;ii++) {//output names, starts, lengths
         chrN<<P->chrName[ii]<<"\n";
         chrS<<P->chrStart[ii]<<"\n";
         chrL<<P->chrLength.at(ii)<<"\n";
@@ -236,7 +186,6 @@ void genomeGenerate(Parameters *P) {
     if (P->GstrandBit<32) P->GstrandBit=32; //TODO: use simple access function for SA
     
     P->GstrandMask = ~(1LLU<<P->GstrandBit);
-    P->nSAbyte=P->nSA*(P->GstrandBit+1)/8+1;
     PackedArray SA1;//SA without sjdb
     SA1.defineBits(P->GstrandBit+1,P->nSA);
     PackedArray SA2;//SA with sjdb, reserve more space
@@ -249,20 +198,21 @@ void genomeGenerate(Parameters *P) {
     };
         
     P->inOut->logMain  << "Number of SA indices: "<< P->nSA << "\n"<<flush;    
-    P->inOut->logMain  << "SA size in bytes: "<< P->nSAbyte << "\n"<<flush;
 
     //sort SA
     time ( &rawTime );
     P->inOut->logMain     << timeMonthDayTime(rawTime) <<" ... starting to sort  Suffix Array. This may take a long time...\n" <<flush;   
     *P->inOut->logStdOut  << timeMonthDayTime(rawTime) <<" ... starting to sort  Suffix Array. This may take a long time...\n" <<flush;
+   
 
-    for (uint ii=0;ii<N;ii++) {//re-fill the array backwards for sorting
-        swap(G[N2-1-ii],G[ii]);
-    };          
-    globalG=G;
-    globalL=L/sizeof(uint);        
-
-    {//not enough RAM, split into chunks          
+//     if (false)
+    {//sort SA chunks
+        
+        for (uint ii=0;ii<N;ii++) {//re-fill the array backwards for sorting
+            swap(G[N2-1-ii],G[ii]);
+        };          
+        globalG=G;
+        globalL=L/sizeof(uint);
         //count the number of indices with 4nt prefix
         uint indPrefN=1LLU << 16;
         uint* indPrefCount = new uint [indPrefN];
@@ -308,7 +258,7 @@ void genomeGenerate(Parameters *P) {
         *P->inOut->logStdOut  << timeMonthDayTime(rawTime) <<" ... sorting Suffix Array chunks and saving them to disk...\n" <<flush;
 
         #pragma omp parallel for num_threads(P->runThreadN) ordered schedule(dynamic,1)
-        for (int iChunk=0; (uint)iChunk < saChunkN; iChunk++) {//start the chunk cycle: sort each chunk with qsort and write to a file
+        for (int iChunk=0; iChunk < (int) saChunkN; iChunk++) {//start the chunk cycle: sort each chunk with qsort and write to a file
             uint* saChunk=new uint [indPrefChunkCount[iChunk]];//allocate local array for each chunk
             for (uint ii=0,jj=0;ii<N2;ii+=P->genomeSAsparseD) {//fill the chunk with SA indices
                 if (G[ii]<4) {
@@ -328,10 +278,9 @@ void genomeGenerate(Parameters *P) {
                 saChunk[ii]=N2-1-saChunk[ii];
             };  
             //write files
-            ofstream saChunkFile;
             string chunkFileName=P->genomeDir+"/SA_"+to_string( (uint) iChunk);
-            ofstrOpen(chunkFileName,"ERROR_00105", P, saChunkFile);   
-            fstreamWriteBig(saChunkFile, (char*) saChunk, sizeof(saChunk[0])*indPrefChunkCount[iChunk],chunkFileName,"ERROR_00121",P);
+            ofstream & saChunkFile = ofstrOpen(chunkFileName,ERROR_OUT, P);   
+            fstreamWriteBig(saChunkFile, (char*) saChunk, sizeof(saChunk[0])*indPrefChunkCount[iChunk],chunkFileName,ERROR_OUT,P);
             saChunkFile.close();
             delete [] saChunk;
             saChunk=NULL;
@@ -405,95 +354,31 @@ void genomeGenerate(Parameters *P) {
 ////////////////////////////////////////
 //          SA index
 //
-    time(&rawTime);    
-    timeString=asctime(localtime ( &rawTime ));
-    timeString.erase(timeString.end()-1,timeString.end());
-    P->inOut->logMain    << timeMonthDayTime(rawTime) <<" ... starting to generate Suffix Array index...\n" <<flush;   
-    *P->inOut->logStdOut << timeMonthDayTime(rawTime) <<" ... starting to generate Suffix Array index...\n" <<flush; 
-        
-    P->genomeSAindexStart = new uint [P->genomeSAindexNbases+1];
-    P->genomeSAindexStart[0]=0;
-    for (uint ii=1;ii<=P->genomeSAindexNbases;ii++) {//L-mer indices starts
-        P->genomeSAindexStart[ii] = P->genomeSAindexStart[ii-1] + ( 1LLU<<(2*ii) );
-    };
-    P->nSAi = P->genomeSAindexStart[P->genomeSAindexNbases];
-    
-    uint* SAi=new uint[P->nSAi];
-//     for (uint isa=0; isa<P->nSAi; isa++) {//initialize
-//         SAi[isa]=P->nSA; //if the suffix is not found in the genome, it's location will be marked with this value
+//     PackedArray SAold;
+// 
+//     if (true)
+//     {//testing: load SA from disk
+//             //read chunks and pack into full SA1
+//         
+//         ifstream oldSAin("./DirTrue/SA");
+//         oldSAin.seekg (0, ios::end);
+//         P->nSAbyte=(uint) oldSAin.tellg();
+//         oldSAin.clear();        
+//         oldSAin.seekg (0, ios::beg);
+// 
+//         P->nSA=(P->nSAbyte*8)/(P->GstrandBit+1);
+//         SAold.defineBits(P->GstrandBit+1,P->nSA);  
+//         SAold.allocateArray();
+//         
+//         oldSAin.read(SAold.charArray,SAold.lengthByte);
+//         oldSAin.close();
+//         
+//         SA1=SAold;
+//         SA2=SAold;
 //     };
     
-    uint* ind0=new uint[P->genomeSAindexNbases];
-    uint* indSAlast=new uint[P->genomeSAindexNbases];
-
-    for (uint ii=0; ii<P->genomeSAindexNbases; ii++) {
-        ind0[ii]=-1;//this is needed in case "AAA...AAA",i.e. indPref=0 is not present in the genome for some lengths
-        indSAlast[ii]=P->nSA;//that's probably not needed
-    };
-
-    P->SAiMarkNbit=P->GstrandBit+1;
-    P->SAiMarkAbsentBit=P->GstrandBit+2;
-    
-    P->SAiMarkNmaskC=1LLU << P->SAiMarkNbit;
-    P->SAiMarkNmask=~P->SAiMarkNmaskC;
-    P->SAiMarkAbsentMaskC=1LLU << P->SAiMarkAbsentBit;
-    P->SAiMarkAbsentMask=~P->SAiMarkAbsentMaskC;
-       
-    
-    for (uint isa=0; isa<P->nSA; isa++) {//for all suffixes
-        if (isa%100000000==0) P->inOut->logMain  << isa*100/P->nSA << "% " << flush;         
-        
-        uint SAstr=SA1[isa];
-        bool dirG = (SAstr>>P->GstrandBit) == 0; //forward or reverse strand of the genome
-        SAstr &= P->GstrandMask;
-        if (!dirG) SAstr=P->nGenome-1-SAstr;
-
-        uint indPref=0;
-        for (uint iL=0; iL < P->genomeSAindexNbases; iL++) {//calculate index
-
-            indPref <<= 2;
-            
-            uint g1= (uint) G[dirG ? SAstr+iL : SAstr-iL]; //reverese if (-) strand
-
-            if (g1>3) {//if N, this suffix does not belong in SAi
-                for (uint iL1=iL; iL1 < P->genomeSAindexNbases; iL1++) {
-                    SAi[P->genomeSAindexStart[iL1]+ind0[iL1]] |= P->SAiMarkNmaskC;
-                };
-                break;
-            };
-
-            if (!dirG) g1=3-g1; //complement if (-) strand
-
-            indPref += (uint) g1;
-            
-            if ( indPref > ind0[iL] || isa==0 ) {//new && good index, record it
-                SAi[P->genomeSAindexStart[iL]+indPref]=isa;
-                for (uint ii=ind0[iL]+1; ii<indPref; ii++) {//index is not present, record to the last present suffix
-                    SAi[P->genomeSAindexStart[iL]+ii] = isa | P->SAiMarkAbsentMaskC; 
-                };
-                ind0[iL]=indPref;
-//                 indSAlast[iL]=isa;
-//             } else if (indPref==ind0[iL]) {
-//                 indSAlast[iL]=isa;//last SA index with the same prefix 
-            } else if ( indPref < ind0[iL] ) {
-                ostringstream errOut;
-                errOut << "BUG: next index is smaller than previous, EXITING\n" <<flush;
-                exitWithError(errOut.str(),std::cerr, P->inOut->logMain, EXIT_CODE_INPUT_FILES, *P);
-            };
-        };
-    };//for (uint isa=0; isa<P->nSA; isa++)
-    P->inOut->logMain << " done\n"<<flush;
-   
-    //pack SAi
     PackedArray SAip;
-    SAip.defineBits(P->GstrandBit+3,P->nSAi);//SAi uses an extra bit compared to SA because it needs to store values > nSA
-    SAip.pointArray((char*) SAi);
-    for (uint ii=0;ii<SAip.length;ii++) {
-        SAip.writePacked(ii,SAi[ii]);
-    };
-    
-    delete [] indSAlast;
-    delete [] ind0;
+    genomeSAindex(G,SA1,P,SAip);
 
     if (P->sjdbFileChrStartEnd.at(0)!="-" || P->sjdbGTFfile!="-")
     {//insert junctions
@@ -515,7 +400,8 @@ void genomeGenerate(Parameters *P) {
         
         //write an extra 0 at the end of the array, filling the last bytes that otherwise are not accessible, but will be written to disk
         //this is - to avoid valgrind complaints. Note that SA2 is allocated with plenty of space to spare.
-        SA2.writePacked(P->nSA,0);
+        SA1=mainGenome.SA;
+        SA1.writePacked(P->nSA,0);
     };
     
     //write genome to disk
@@ -523,19 +409,19 @@ void genomeGenerate(Parameters *P) {
     P->inOut->logMain     << timeMonthDayTime(rawTime) <<" ... writing Genome to disk ...\n" <<flush;   
     *P->inOut->logStdOut  << timeMonthDayTime(rawTime) <<" ... writing Genome to disk ...\n" <<flush;   
     
-    ofstream genomeOut;
-    ofstrOpen(P->genomeDir+"/Genome","ERROR_00104", P, genomeOut);   
-    fstreamWriteBig(genomeOut,G,P->nGenome,P->genomeDir+"/Genome","ERROR_00120",P);
+    ofstream & genomeOut = ofstrOpen(P->genomeDir+"/Genome",ERROR_OUT, P);   
+    fstreamWriteBig(genomeOut,G,P->nGenome,P->genomeDir+"/Genome",ERROR_OUT,P);
     genomeOut.close();  
 
     //write SA                
     time ( &rawTime );
+    P->inOut->logMain  << "SA size in bytes: "<<SA1.lengthByte << "\n"<<flush;
+
     P->inOut->logMain     << timeMonthDayTime(rawTime) <<" ... writing Suffix Array to disk ...\n" <<flush;   
     *P->inOut->logStdOut  << timeMonthDayTime(rawTime) <<" ... writing Suffix Array to disk ...\n" <<flush;   
 
-    ofstream SAout;
-    ofstrOpen(P->genomeDir+"/SA","ERROR_00106", P, SAout);   
-    fstreamWriteBig(SAout,(char*) SA2.charArray, (streamsize) P->nSAbyte,P->genomeDir+"/SA","ERROR_00122",P);
+    ofstream & SAout = ofstrOpen(P->genomeDir+"/SA",ERROR_OUT, P);   
+    fstreamWriteBig(SAout,(char*) SA1.charArray, (streamsize) SA1.lengthByte,P->genomeDir+"/SA",ERROR_OUT,P);
     SAout.close();    
     
     //write SAi
@@ -544,16 +430,14 @@ void genomeGenerate(Parameters *P) {
     *P->inOut->logStdOut << timeMonthDayTime(rawTime) <<" ... writing SAindex to disk\n" <<flush;   
     
     //write SAi to disk
-    ofstream SAiOut;
-    ofstrOpen(P->genomeDir+"/SAindex","ERROR_00107", P, SAiOut);   
+    ofstream & SAiOut = ofstrOpen(P->genomeDir+"/SAindex",ERROR_OUT, P);
 
-    fstreamWriteBig(SAiOut, (char*) &P->genomeSAindexNbases, sizeof(P->genomeSAindexNbases),P->genomeDir+"/SAindex","ERROR_00123",P);
-    fstreamWriteBig(SAiOut, (char*) P->genomeSAindexStart, sizeof(P->genomeSAindexStart[0])*(P->genomeSAindexNbases+1),P->genomeDir+"/SAindex","ERROR_00124",P);        
-    fstreamWriteBig(SAiOut,  SAip.charArray, SAip.lengthByte,P->genomeDir+"/SAindex","ERROR_00125",P);
+    fstreamWriteBig(SAiOut, (char*) &P->genomeSAindexNbases, sizeof(P->genomeSAindexNbases),P->genomeDir+"/SAindex",ERROR_OUT,P);
+    fstreamWriteBig(SAiOut, (char*) P->genomeSAindexStart, sizeof(P->genomeSAindexStart[0])*(P->genomeSAindexNbases+1),P->genomeDir+"/SAindex",ERROR_OUT,P);        
+    fstreamWriteBig(SAiOut,  SAip.charArray, SAip.lengthByte,P->genomeDir+"/SAindex",ERROR_OUT,P);
     SAiOut.close();    
 
     SA2.deallocateArray();
-    delete [] SAi;    
 
     time(&rawTime);
     timeString=asctime(localtime ( &rawTime ));
